# pylint: disable=too-many-ancestors
"""Treatment and conversion of de timestamp."""

from datetime import datetime
from zoneinfo import ZoneInfo

# Set UTC as default
UTC = ZoneInfo("UTC")


class UnixTimestamp:
    """Class to handle Unix timestamp conversions."""

    @staticmethod
    def current_timestamp(timezone: ZoneInfo = UTC) -> int:
        """
        Get the current Unix timestamp in seconds.

        Args:
            timezone (ZoneInfo, optional): Timezone to use. Defaults to UTC.

        Returns:
            int: Current Unix timestamp.

        """
        return int(datetime.now(timezone).timestamp())

    @staticmethod
    def to_datetime(timestamp: int, timezone: ZoneInfo = UTC) -> datetime:
        """
        Convert a Unix timestamp (integer) into a datetime object.

        Args:
            timestamp (int): Unix timestamp in seconds.
            timezone (ZoneInfo, optional): Timezone to convert to. Defaults to UTC.

        Returns:
            datetime: Datetime object in the specified timezone.

        """
        return datetime.fromtimestamp(timestamp, timezone)

    @staticmethod
    def from_datetime(dt: datetime) -> int:
        """
        Convert a datetime object to a Unix timestamp.

        Args:
            dt (datetime): Datetime object.

        Returns:
            int: Unix timestamp (seconds since epoch).

        """
        return int(dt.timestamp())



from datetime import datetime
from typing import Optional

from sqlalchemy import DateTime, func
from sqlalchemy.orm import DeclarativeBase, Mapped, Session, mapped_column


class BaseModel(DeclarativeBase):
    """Base class for all models."""

    __abstract__ = True

    deleted_at: Mapped[Optional[datetime]] = mapped_column(DateTime, nullable=True)
    created_at: Mapped[datetime] = mapped_column(
        DateTime,
        server_default=func.now(),  # pylint: disable=not-callable
        nullable=False,
    )
    updated_at: Mapped[datetime] = mapped_column(
        DateTime,
        server_default=func.now(),  # pylint: disable=not-callable
        onupdate=func.now(),  # pylint: disable=not-callable
        nullable=False,
    )

    def soft_delete(self, session: Session) -> None:
        """
        Marks the record as deleted by setting the `deleted_at` timestamp.

        Args:
            session (Session): The SQLAlchemy session to use for the operation.
        """
        self.deleted_at = datetime.now()
        session.add(self)
        session.flush()



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from app.config.settings import settings

engine = create_engine(settings.DATABASE_URL)

SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)


def get_db():
    """
    Yields a database session. Commits after the nested block completes, or rolls back
    if an exception occurs. Always closes the session when done.
    """
    db = SessionLocal()
    try:
        yield db
        db.commit()
    except Exception:
        db.rollback()
        raise
    finally:
        db.close()
